/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include "CvfdColor.h"

using namespace std;
using namespace FD;

class CvfdRectangle;
   
DECLARE_NODE(CvfdRectangle)
/*Node
 *
 * @name CvfdRectangle
 * @category FDOPENCV_ORG:Draw
 * @description Draws a rectangle in the image
 *
 * @input_name IMAGEIN
 * @input_description the input image
 * @input_type CvfdImage
 *
 * @input_name COLOR
 * @input_description the BGR color of the rectangle
 * @input_type CvfdColor
 *
 * @parameter_name X1
 * @parameter_type int
 * @parameter_value 0
 * @parameter_description x of the rectangle vertices   
 *
 * @parameter_name Y1
 * @parameter_type int
 * @parameter_value -1
 * @parameter_description y of the rectangle vertices
 *
 * @parameter_name X2
 * @parameter_type int
 * @parameter_value -1
 * @parameter_description x opposite rectangle vertex
 *   
 * @parameter_name Y2
 * @parameter_type int
 * @parameter_value -1
 * @parameter_description y opposite rectangle vertex   
 *
 * @parameter_name THICKNESS
 * @parameter_type int
 * @parameter_value -1
 * @parameter_description Thickness of lines that make up the rectangle. Negative values, make the function to draw a filled rectangle
 *
 * @parameter_name LINE_TYPE
 * @parameter_type string
 * @parameter_value 8;4;CV_AA
 * @parameter_description Type of the contour segments
 *
 * @parameter_name SHIFT
 * @parameter_type int
 * @parameter_value 0
 * @parameter_description Number of fractional bits in the point coordinates
 *
 * @output_name IMAGEOUT
 * @output_description The result image
 * @output_type CvfdImage
 *
 END*/
inline std::string CINT(int value)
{
  std::stringstream sstr;
  sstr << value;
  return(sstr.str());
}   
   
class CvfdRectangle : public BufferedNode {
      
  //Input ID
  int m_colorID;
  int m_imageInID;
  //Output ID
  int m_imageOutID;
  //Parameter
  int m_x1;
  int m_y1;
  int m_x2;
  int m_y2;      
  string m_lineType;
  int m_thickness;
  int m_shift; 
  //map of the line_type
  map<string,int> m_lineTypeMap;
      
public:
  CvfdRectangle(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    //add inputs
    m_colorID = addInput("COLOR");
    m_imageInID = addInput("IMAGEIN");
         
    //add outputs
    m_imageOutID = addOutput("IMAGEOUT");
         
    //Initialize parameters         
    m_x1 = dereference_cast<int>(parameters.get("X1"));
    m_y1 = dereference_cast<int>(parameters.get("Y1"));
    m_x2 = dereference_cast<int>(parameters.get("X2"));
    m_y2 = dereference_cast<int>(parameters.get("Y2"));
    m_thickness = dereference_cast<int>(parameters.get("THICKNESS"));
    m_shift = dereference_cast<int>(parameters.get("SHIFT"));         
    m_lineType = object_cast<String>(parameters.get("LINE_TYPE"));         
         
    //Initialize lineType map
    m_lineTypeMap["8"] = 8;
    m_lineTypeMap["4"] = 4;
    m_lineTypeMap["CV_AA"] = CV_AA;         
  }
      
  void calculate(int output_id, int count, Buffer &out)
  {           
    //Read the inputs
    RCPtr<CvfdColor> colorPtr = getInput(m_colorID,count);
    RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count);
         
    //Handle
    CvfdImage* image = new CvfdImage(imagePtr->getImage());
         
    int status = cvGetErrMode();
    cvSetErrMode( CV_ErrModeSilent );
    __BEGIN__; 
    OPENCV_CALL( cvRectangle( image->getImage(), cvPoint(m_x1,m_y1)
			      , cvPoint(m_x2,m_y2), CV_RGB( (colorPtr->getColor()).val[0], (colorPtr->getColor()).val[1], (colorPtr->getColor()).val[2] )
			      , m_thickness ,m_lineTypeMap[m_lineType], m_shift ));         
    __END__;
    cvSetErrMode( status );
    if( cvGetErrStatus() != CV_StsOk  )
      {
	throw new GeneralException("OPENCV - Error to draw a rectangle: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
      } 
         
    out[count] = ObjectRef(image);
  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdRectangle)
};
